﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectWarmlyShip.Entities;

namespace ProjectWarmlyShip.DrawningObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningShip
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityShip? EntityShip { get; protected set; }
        private IDrawingDecks? DrawingDecks;
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки автомобиля
        /// </summary>
        protected int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки автомобиля
        /// </summary>
        protected int _startPosY;
        /// <summary>
        /// Ширина прорисовки автомобиля
        /// </summary>
        protected readonly int _shipWidth = 200;
        /// <summary>
        /// Высота прорисовки автомобиля
        /// </summary>
        public int _shipHeight = 30;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawningShip(int speed, double weight, Color bodyColor, int width, int height, int numDecks, int nummode)
        {
            // TODO: Продумать проверки
            if (width > _shipWidth && height > _shipHeight)
            {
                _pictureWidth = width;
                _pictureHeight = height;
                EntityShip = new EntityShip(speed, weight, bodyColor);
                int mode = nummode % 3;
                switch (mode)
                {
                    case 0:
                        DrawingDecks = new DrawDecksSquare();
                        break;
                    case 1:
                        DrawingDecks = new DrawDecksTrapeze();
                        break;
                    case 2:
                        DrawingDecks = new DrawDecksParal();
                        break;
                }
                DrawingDecks.SomeProperty = numDecks;
                if (numDecks == 2)
                    _shipHeight = 42;
                if (numDecks == 3)
                    _shipHeight = 54;
            }
        }
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="shipWidth">Ширина прорисовки автомобиля</param>
        /// <param name="shipHeight">Высота прорисовки автомобиля</param>
        protected DrawningShip(int speed, double weight, Color bodyColor, int width, int height, int shipWidth, int shipHeight, int numDecks)
        {
            // TODO: Продумать проверки
            //if (width > _shipWidth && height > _shipHeight)

            _pictureWidth = width;
            _pictureHeight = height;
            _shipWidth = shipWidth;
            _shipHeight = shipHeight;
            EntityShip = new EntityShip(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if ((x > 0) && (x < _pictureWidth))
                _startPosX = x;
            else _startPosX = 0;
            if ((y > 0) && (y < _pictureHeight))
                _startPosY = y;
            else _startPosY = 0;
            _startPosX = x;
            _startPosY = y;
        }
        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _shipWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _shipHeight;
        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityShip == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityShip.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityShip.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + _shipWidth + EntityShip.Step < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + _shipHeight + EntityShip.Step < _pictureHeight,
            };
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityShip == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityShip.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityShip.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityShip.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityShip.Step;
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityShip == null)
            {
                return;
            }
            Brush BodyBrush = new SolidBrush(EntityShip.BodyColor);

            //границы корабля

            if (_shipHeight == 30)
            {
                Point point1 = new Point(_startPosX, _startPosY + 12);
                Point point2 = new Point(_startPosX + 25, _startPosY + 35);
                Point point3 = new Point(_startPosX + 175, _startPosY + 35);
                Point point4 = new Point(_startPosX + 200, _startPosY + 12);

                Point[] curvePoints1 = { point1, point2, point3, point4 };
                g.FillPolygon(BodyBrush, curvePoints1);

            }
            if (_shipHeight == 42)
            {
                Point point1 = new Point(_startPosX, _startPosY + 24);
                Point point2 = new Point(_startPosX + 25, _startPosY + 47);
                Point point3 = new Point(_startPosX + 175, _startPosY + 47);
                Point point4 = new Point(_startPosX + 200, _startPosY + 22);

                Point[] curvePoints1 = { point1, point2, point3, point4 };
                g.FillPolygon(BodyBrush, curvePoints1);

            }
            if (_shipHeight == 54)
            {
                Point point1 = new Point(_startPosX, _startPosY + 36);
                Point point2 = new Point(_startPosX + 25, _startPosY + 59);
                Point point3 = new Point(_startPosX + 175, _startPosY + 59);
                Point point4 = new Point(_startPosX + 200, _startPosY + 34);

                Point[] curvePoints1 = { point1, point2, point3, point4 };
                g.FillPolygon(BodyBrush, curvePoints1);

            }
            DrawingDecks.Draw(_startPosX, _startPosY, EntityShip.BodyColor, g);
        }
    }
}
